@@ -29,7 +29,7 @@ class Document(object):
     This is a immutable class around the text and cursor position, and contains
     methods for querying this data, e.g. to give the text before the cursor.
 
-    This class is usually instantiated by a :class:`~prompt_toolkit.line.Line`
+    This class is usually instantiated by a :class:`~prompt_toolkit.buffer.Buffer`
     object, and accessed as the `document` property of that class.
 
     :param text: string
@@ -475,6 +475,12 @@ class Document(object):
         """ Relative position for the end of this line. """
         return len(self.current_line_after_cursor)
 
+    def last_non_blank_of_current_line_position(self):
+        """
+        Relative position for the last non blank character of this line.
+        """
+        return len(self.current_line_after_cursor.rstrip())
+
     def get_column_cursor_position(self, column):
         """
         Return the relative cursor position for this column at the current
