@@ -7,6 +7,9 @@ __all__ = ('History', 'FileHistory')
 
 
 class History(object):
+    """
+    Base ``History`` class that keeps a list of all strings in memory.
+    """
     def __init__(self):
         self.strings = []
 
@@ -20,8 +23,10 @@ class History(object):
         return len(self.strings)
 
 
-
 class FileHistory(History):
+    """
+    ``History`` class that stores all strings in a file.
+    """
     def __init__(self, filename):
         super(FileHistory, self).__init__()
         self.filename = filename
@@ -51,7 +56,6 @@ class FileHistory(History):
 
                 add()
 
-
     def append(self, string):
         super(FileHistory, self).append(string)
 
@@ -62,5 +66,3 @@ class FileHistory(History):
             write('\n# %s\n' % datetime.datetime.now())
             for line in string.split('\n'):
                 write('+%s\n' % line)
-
-
