@@ -10,7 +10,7 @@ import termios
 import tty
 
 from ..keys import Keys
-from ..key_binding import KeyPress
+from ..key_binding.input_processor import KeyPress
 
 __all__ = (
     'InputStream',
@@ -112,6 +112,10 @@ class InputStream(object):
         '\x1bOQ': Keys.F2,
         '\x1bOR': Keys.F3,
         '\x1bOS': Keys.F4,
+        '\x1b[11~': Keys.F1,  # rxvt-unicode
+        '\x1b[12~': Keys.F2,  # rxvt-unicode
+        '\x1b[13~': Keys.F3,  # rxvt-unicode
+        '\x1b[14~': Keys.F4,  # rxvt-unicode
         '\x1b[15~': Keys.F5,
         '\x1b[17~': Keys.F6,
         '\x1b[18~': Keys.F7,
@@ -128,6 +132,14 @@ class InputStream(object):
         '\x1b[32~': Keys.F18,
         '\x1b[33~': Keys.F19,
         '\x1b[34~': Keys.F20,
+        '\x1b[1;5D': Keys.ControlLeft,   # Cursor Mode
+        '\x1b[1;5C': Keys.ControlRight,  # Cursor Mode
+        '\x1b[1;5A': Keys.ControlUp,     # Cursor Mode
+        '\x1b[1;5B': Keys.ControlDown,   # Cursor Mode
+        '\x1bOD': Keys.ControlLeft,     # Application Mode (tmux)
+        '\x1bOC': Keys.ControlRight,    # Application Mode (tmux)
+        '\x1bOA': Keys.ControlUp,       # Application Mode (tmux)
+        '\x1bOB': Keys.ControlDown,     # Application Mode (tmux)
 
         # Meta + arrow keys. Several terminals handle this differently.
         # The following sequences are for xterm and gnome-terminal.
