@@ -1,9 +1,9 @@
 from ctypes import windll, pointer, c_int
 from ctypes.wintypes import DWORD
 
-from ..keys import Keys
-from ..key_binding import KeyPress
-from ..win32_types import EventTypes, KEY_EVENT_RECORD, INPUT_RECORD, STD_INPUT_HANDLE
+from prompt_toolkit.key_binding.input_processor import KeyPress
+from prompt_toolkit.keys import Keys
+from prompt_toolkit.win32_types import EventTypes, KEY_EVENT_RECORD, INPUT_RECORD, STD_INPUT_HANDLE
 
 __all__ = (
     'ConsoleInputReader',
@@ -140,6 +140,21 @@ class ConsoleInputReader(object):
             else:
                 result = KeyPress(ev.AsciiChar.decode('ascii'), ev.AsciiChar.decode('ascii'))
 
+        # Correctly handle Control-Arrow keys.
+        if (ev.ControlKeyState & self.LEFT_CTRL_PRESSED or
+                ev.ControlKeyState & self.RIGHT_CTRL_PRESSED) and result:
+            if result.key == Keys.Left:
+                 result.key = Keys.ControlLeft
+
+            if result.key == Keys.Right:
+                 result.key = Keys.ControlRight
+
+            if result.key == Keys.Up:
+                 result.key = Keys.ControlUp
+
+            if result.key == Keys.Down:
+                 result.key = Keys.ControlDown
+
         # Turn 'Tab' into 'BackTab' when shift was pressed.
         if ev.ControlKeyState & self.SHIFT_PRESSED and result:
             if result.key == Keys.Tab:
